open Pts
open Format


let usage=sprintf "usage : %s <srcfile>" Sys.executable_name
 
let theory_file=
  match
    let accu=ref [] in
    let f str=accu:=str::!accu in
      Arg.parse [] f usage;!accu
  with
      [a]->a
    |_->(Arg.usage [] usage;exit 0) 

let read_data myparse mylex file=
  let chan=open_in file in
  let buf=Lexing.from_channel chan in
  let data=try myparse mylex buf 
  with Parsing.Parse_error | Failure "lexing: empty token"->
    let cpos=(Lexing.lexeme_start buf)- !Ptslex.bline in 
      failwith 
	(sprintf "Syntax error in %s line %d char %d" 
	   file !Ptslex.nline cpos)
  in close_in chan;data

let (basename,parse_pts)=
    read_data Ptsparse.parse_all Ptslex.token theory_file
    
let varname="PTS_SYS_PATH"

let libpath=
  try (Sys.getenv varname) with 
    Not_found->failwith (sprintf "Environment variable %s undefined" varname)

let sys_file=basename^".sys"

let parse_sys=
  let filename=
    if Sys.file_exists sys_file then 
      sys_file
    else 
      let libfile=Filename.concat libpath sys_file in
	if Sys.file_exists libfile then libfile 
	else 
	  failwith (sprintf "System %s not found" basename) in
    read_data Ptsparse.parse_sys Ptslex.systoken filename
      
let _=Mksys.init parse_sys
let env=MY_PTS.read_prog parse_pts
let _=MY_PTS.dump env


