
type statement = Var | Def | Mac

module type SIGN =
  sig
    type s
    val axiom : s -> s option
    val rule : s -> s -> s option
    val of_string : string -> s option
    val to_string : s -> string
  end

module PTS :
  functor (MY_SIGN : SIGN) ->
    sig
      type t =
          Sort of MY_SIGN.s
        | Dum
        | Rel of int
        | Lam of string * t * t
        | Prod of string * t * t
        | App of t * t
      and raw =
          RDum
        | RVar of string
        | RLam of string * raw * raw
        | RProd of string * raw * raw
        | RApp of raw * raw
      and env = (string * t * t option) list
      val push : env -> string -> t -> (string * t * t option) list
      val translocate : int -> t -> t
      val search_var : string -> env -> t
      val search_env : int -> env -> string * t * t option
      val search_name : int -> env -> string option
      val search_type : int -> env -> t
      val search_body : int -> env -> t option
      val unname : env -> raw -> t
      val subst : t -> t -> t
      val head_reduce : env -> t -> t
      val mac_reduce : env -> t -> t
      val reduce : env -> t -> t
      val conv : env -> t -> t -> bool
      val type_of : env -> t -> t
      val push_stat :
        env -> string * raw * statement -> (string * t * t option) list
      val print : env -> t -> unit
      val print_mac : env -> t -> unit
      val print_stat : env -> string * t * t option -> unit
      val dump : env -> unit
      val read_prog : (string * raw * statement) list -> env
    end

module MY_PTS :
  sig
    type t =
      PTS(Mksys).t =
        Sort of Mksys.s
      | Dum
      | Rel of int
      | Lam of string * t * t
      | Prod of string * t * t
      | App of t * t
    and raw =
      PTS(Mksys).raw =
        RDum
      | RVar of string
      | RLam of string * raw * raw
      | RProd of string * raw * raw
      | RApp of raw * raw
    and env = (string * t * t option) list
    val push : env -> string -> t -> (string * t * t option) list
    val translocate : int -> t -> t
    val search_var : string -> env -> t
    val search_env : int -> env -> string * t * t option
    val search_name : int -> env -> string option
    val search_type : int -> env -> t
    val search_body : int -> env -> t option
    val unname : env -> raw -> t
    val subst : t -> t -> t
    val head_reduce : env -> t -> t
    val mac_reduce : env -> t -> t
    val reduce : env -> t -> t
    val conv : env -> t -> t -> bool
    val type_of : env -> t -> t
    val push_stat :
      env -> string * raw * statement -> (string * t * t option) list
    val print : env -> t -> unit
    val print_mac : env -> t -> unit
    val print_stat : env -> string * t * t option -> unit
    val dump : env -> unit
    val read_prog : (string * raw * statement) list -> env

  end
