%{ 
open Pts

let rec mkabs t=function 
    []->t
  | nam::q -> MY_PTS.RLam (nam,MY_PTS.RDum,mkabs t q)

%}

%token <string> IDENT
%token COLON
%token ARROW
%token LPAR
%token RPAR
%token LBRA
%token RBRA
%token LACC
%token RACC
%token DOT
%token DEF
%token ASSIGN
%token END
%token VAR
%token MAC
%token COMMA
%token SEMICOLON
%token SORTS
%token AXIOMS
%token RULES
%token SYSTEM

%start parse_all
%start parse_sys
%start sterm
%type <string*(string*Pts.MY_PTS.raw*Pts.statement) list> parse_all
%type <Pts.MY_PTS.raw> sterm
%type <(string list)*((string*string) list)*(((string*string)*string) list)> parse_sys

%%

/* Rgles de grammaire et actions smantiques */

parse_all:
  SYSTEM IDENT body {$2,$3}
;

body: 
   decl body                     {$1::$2}
 | END {[]}
;

decl: 
   DEF IDENT ASSIGN sterm {($2,$4,Pts.Def)}
 | MAC IDENT args ASSIGN sterm {($2,mkabs $5 $3,Pts.Mac)}  
 | VAR IDENT COLON sterm {($2,$4,Pts.Var)}
;

args:
   IDENT args  {$1::$2}
 | IDENT {[$1]}
;

litteral :
   IDENT {Pts.MY_PTS.RVar $1}
;

arrow :
   LPAR application RPAR ARROW term {Pts.MY_PTS.RProd("_",$2,$5)}
 | litteral ARROW term {Pts.MY_PTS.RProd("_",$1,$3)}
;

application:
 | term {$1}
 | application term {Pts.MY_PTS.RApp ($1,$2)}
;

term :
 | arrow {$1}
 | LPAR IDENT COLON term RPAR term {Pts.MY_PTS.RProd($2,$4,$6)}
 | LBRA IDENT COLON term RBRA term {Pts.MY_PTS.RLam($2,$4,$6)}
 | litteral {$1}
 | LPAR application RPAR {$2}
;

sterm:
application DOT {$1}

/* fichiers de dfinition de PTS */

parse_sys: 
   SORTS sorts AXIOMS axioms RULES rules END {($2,$4,$6)}
;

sorts : 
  IDENT sorts {$1::$2}
| {[]}
;

axioms:
  axiom axioms {$1::$2}
|  {[]}
;

axiom:
  IDENT COLON IDENT {($1,$3)}
;

rules:
  rule rules {$1::$2}
|  {[]}
;

rule:
  IDENT ARROW IDENT COLON IDENT {(($1,$3),$5)}
| IDENT ARROW IDENT {(($1,$3),$3)}
;
